@@ -2,7 +2,7 @@
 
 ;; Copyright (C) 1989-1997 D. Bates, Kademan, Ritter, D.M. Smith, K. Hornik,
 ;;      R.M. Heiberger, M. Maechler, and A.J. Rossini.
-;; Copyright (C) 1998-2005 A.J. Rossini, Rich M. Heiberger, Martin
+;; Copyright (C) 1998-2005 A.J. Rossini, Richard M. Heiberger, Martin
 ;;      Maechler, Kurt Hornik, Rodney Sparapani, and Stephen Eglen.
 
 ;; Author: A.J. Rossini <rossini@biostat.washington.edu>
@@ -37,43 +37,43 @@
 
  ; Configuration variables
 
-(defvar S-syntax-table nil "Syntax table for S code.")
-(if S-syntax-table
-    nil
-  (setq S-syntax-table (make-syntax-table))
-  (modify-syntax-entry ?\\ "\\" S-syntax-table)
-  (modify-syntax-entry ?+  "."  S-syntax-table)
-  (modify-syntax-entry ?-  "."  S-syntax-table)
-  (modify-syntax-entry ?=  "."  S-syntax-table)
-  (modify-syntax-entry ?%  "."  S-syntax-table)
-  (modify-syntax-entry ?<  "."  S-syntax-table)
-  (modify-syntax-entry ?>  "."  S-syntax-table)
-  (modify-syntax-entry ?&  "."  S-syntax-table)
-  (modify-syntax-entry ?|  "."  S-syntax-table)
-  (modify-syntax-entry ?\' "\"" S-syntax-table)
-  (modify-syntax-entry ?\" "\"" S-syntax-table)
-  (modify-syntax-entry ?#  "<"  S-syntax-table) ; open comment
-  (modify-syntax-entry ?\n ">"  S-syntax-table) ; close comment
-  ;;(modify-syntax-entry ?.  "w"  S-syntax-table) ; "." used in S obj names
-  (modify-syntax-entry ?.  "_"  S-syntax-table) ; see above/below,
+(defvar S-syntax-table
+  (let ((S-syntax-table (make-syntax-table)))
+    (modify-syntax-entry ?\\ "\\" S-syntax-table)
+    (modify-syntax-entry ?+  "."  S-syntax-table)
+    (modify-syntax-entry ?-  "."  S-syntax-table)
+    (modify-syntax-entry ?=  "."  S-syntax-table)
+    (modify-syntax-entry ?%  "."  S-syntax-table)
+    (modify-syntax-entry ?<  "."  S-syntax-table)
+    (modify-syntax-entry ?>  "."  S-syntax-table)
+    (modify-syntax-entry ?&  "."  S-syntax-table)
+    (modify-syntax-entry ?|  "."  S-syntax-table)
+    (modify-syntax-entry ?\' "\"" S-syntax-table)
+    (modify-syntax-entry ?\" "\"" S-syntax-table)
+    (modify-syntax-entry ?#  "<"  S-syntax-table) ; open comment
+    (modify-syntax-entry ?\n ">"  S-syntax-table) ; close comment
+    ;;(modify-syntax-entry ?.  "w"  S-syntax-table) ; "." used in S obj names
+    (modify-syntax-entry ?.  "_"  S-syntax-table) ; see above/below,
                                         ; plus consider separation.
-  (modify-syntax-entry ?$  "_"  S-syntax-table); foo$comp = 1 symbol(completion)
-  (modify-syntax-entry ?@  "_"  S-syntax-table); foo@slot = 1 symbol(completion)
-  (modify-syntax-entry ?_  "."  S-syntax-table)
-  (modify-syntax-entry ?*  "."  S-syntax-table)
-  (modify-syntax-entry ?<  "."  S-syntax-table)
-  (modify-syntax-entry ?>  "."  S-syntax-table)
-  (modify-syntax-entry ?/  "."  S-syntax-table))
+    (modify-syntax-entry ?$  "_"  S-syntax-table); foo$comp = 1 symbol(completion)
+    (modify-syntax-entry ?@  "_"  S-syntax-table); foo@slot = 1 symbol(completion)
+    (modify-syntax-entry ?_  "_"  S-syntax-table)
+    (modify-syntax-entry ?:  "_"  S-syntax-table)
+    (modify-syntax-entry ?*  "."  S-syntax-table)
+    (modify-syntax-entry ?<  "."  S-syntax-table)
+    (modify-syntax-entry ?>  "."  S-syntax-table)
+    (modify-syntax-entry ?/  "."  S-syntax-table)
+    S-syntax-table)
+  "Syntax table for S code."
+  )
 
+
+;; what is R doing here?
 (defvar R-editing-alist
   '((paragraph-start              . (concat "\\s-*$\\|" page-delimiter))
     (paragraph-separate           . (concat "\\s-*$\\|" page-delimiter))
     (paragraph-ignore-fill-prefix . t)
     (require-final-newline        . t)
-    (comment-start                . "#")
-    (comment-add                  . 1)
-    (comment-start-skip           . "#+ *")
-    (comment-column               . 40)
     ;;(comment-indent-function  . 'S-comment-indent)
     ;;(ess-comment-indent           . 'S-comment-indent)
     ;;(ess-indent-line                      . 'S-indent-line)
@@ -81,47 +81,63 @@
     (indent-line-function         . 'ess-indent-line)
     (parse-sexp-ignore-comments   . t)
     (ess-style                    . ess-default-style)
-    (ess-local-process-name       . nil)
     ;;(ess-keep-dump-files          . 'ask)
     (ess-mode-syntax-table        . S-syntax-table)
     ;; For Changelog add, require ' ' before <- : "attr<-" is a function name :
     (add-log-current-defun-header-regexp . "^\\(.+\\)\\s-+<-[ \t\n]*function")
-    (font-lock-defaults           . '(ess-R-mode-font-lock-keywords
+    (ess-font-lock-keywords       . 'ess-R-font-lock-keywords)
+    (ess-font-lock-defaults       . (ess--extract-default-fl-keywords ess-R-font-lock-keywords))
+    (font-lock-defaults           . '(ess-font-lock-defaults
                                       nil nil ((?\. . "w") (?\_ . "w"))))
     )
   "General options for R source files.")
 
+
 (defvar S-editing-alist
   ;; copy the R-list and modify :
   (let ((S-alist (copy-alist R-editing-alist)))
-    (setcdr (assoc 'font-lock-defaults S-alist)
-            (quote '(ess-S-mode-font-lock-keywords nil nil ((?\. . "w")))))
-    ;;      ^^ extra quote is needed - why?
+    (setcdr (assoc 'ess-font-lock-defaults S-alist)
+            '(ess--extract-default-fl-keywords ess-S-font-lock-keywords))
+    (setcdr (assoc 'ess-font-lock-keywords S-alist)
+            (quote 'ess-S-font-lock-keywords))
     S-alist)
   "General options for editing S and S+ source files.")
 
 (defvar inferior-S-language-start
   '(concat "options("
            "STERM='"    ess-STERM  "'"
-           ", str.dendrogram.last =\"'\""
+           ", str.dendrogram.last=\"'\""
            (if ess-editor (concat ", editor='" ess-editor "'"))
            (if ess-pager  (concat ", pager='"  ess-pager  "', help.pager='"  ess-pager  "'"))
+           ", show.error.locations=TRUE"
            ")")
   "S language expression for startup -- default for all S dialects.")
 
 (defconst S-common-cust-alist
-  `((ess-language                  . "S")
+  '((ess-language                  . "S")
     (inferior-ess-exit-command     . "q()\n")
     (inferior-ess-language-start   . (eval inferior-S-language-start))
     (comint-use-prompt-regexp      . t)  ;;use fields if nil
+    (comint-process-echoes	   . t)
     ;; these prompt are the same for all S-languages As long as custom prompt
     ;; ends in inferior-ess-primary-prompt everything should work as expected.
     (inferior-ess-primary-prompt   . "> ")
-    (inferior-ess-secondary-prompt . "+ ")
-    ;; inferior-ess-prompt is used by comint for navigation only if
-    ;; comint-use-prompt-regexp is t transcript-mode also relies on this regexp
+    ;; (inferior-ess-secondary-prompt . "[+:] ") ;; catch Selection: and alike
+    (inferior-ess-secondary-prompt . "+ ") ;; catch Selection: and alike
+    (comment-start                . "#")
+    (ess-imenu-generic-expression  . ess-imenu-S-generic-expression)
+    (comment-add                  . 1)
+    (comment-start-skip           . "#+ *")
+    (comment-use-syntax           . nil) ;; regexp based, probably faster than syntax based
+    (comment-column               . 40)
+    (ess-no-skip-regexp           . (concat "^ *@\\|" (default-value 'ess-no-skip-regexp)))
+    ;; inferior-ess-prompt is used by comint for navigation, only if
+    ;; comint-use-prompt-regexp is t; (transcript-mode also relies on this regexp)
     (inferior-ess-prompt           . inferior-S-prompt) ;customizable
-    (ess-get-help-topics-function  . 'ess-get-S-help-topics)
+    (ess-get-help-topics-function  . 'ess-get-S-help-topics-function)
+    (ess-getwd-command          . "getwd()\n")
+    (ess-setwd-command          . "setwd('%s')\n")
+    (ess-funargs-command        . ".ess.funargs(%s)\n")
     )
   "S-language common settings for all <dialect>-customize-alist s"
   )
@@ -149,7 +165,8 @@
      (ess-syntax-error-re
       . "\\(Syntax error: .*\\) at line \\([0-9]*\\), file \\(.*\\)$")
      (inferior-ess-objects-command  . inferior-Splus-objects-command)
-     (inferior-ess-font-lock-keywords . inferior-ess-S-font-lock-keywords)
+     (ess-describe-object-at-point-commands . 'ess-S-describe-object-at-point-commands)
+     (inferior-ess-font-lock-keywords . 'inferior-S-font-lock-keywords)
      (ess-editor . S-editor)
      (ess-pager  . S-pager)
      )
@@ -668,12 +685,20 @@ typically \" <- \", can be customized.  In ESS modes other than R/S,
 an underscore is always inserted. "
   (interactive)
   ;;(insert (if (ess-inside-string-or-comment-p (point)) "_" ess-S-assign))
-  (if (or
-       (ess-inside-string-or-comment-p (point))
-       (not (equal ess-language "S")))
-      (insert "_")
-    ;; else:
-    (ess-insert-S-assign)))
+  (save-restriction
+    (ignore-errors
+      (when (and (eq major-mode 'inferior-ess-mode)
+                 (> (point) (process-mark (get-buffer-process (current-buffer)))))
+        (narrow-to-region (process-mark (get-ess-process)) (point-max)))
+      (and ess-noweb-mode
+           (ess-noweb-in-code-chunk)
+           (ess-noweb-narrow-to-chunk)))
+    (if (or
+         (ess-inside-string-or-comment-p (point))
+         (not (equal ess-language "S")))
+        (insert "_")
+      ;; else:
+      (ess-insert-S-assign))))
 
 
 (defun ess-insert-S-assign ()
@@ -705,8 +730,6 @@ In that case, the it is removed and replaced by the underscore.
 
  Using \"C-q _\" will always just insert the underscore character."
   (interactive "P")
-  (require 'ess-mode)
-  (require 'ess-inf)
   (let ((current-key (lookup-key ess-mode-map "_")))
     (if (and current-key
              ;; (stringp current-key) (string= current-key ess-S-assign)
@@ -731,9 +754,10 @@ In that case, the it is removed and replaced by the underscore.
   (define-key ess-mode-map          "\C-cf" 'ess-insert-function-outline)
   (define-key inferior-ess-mode-map "\C-cw" 'ess-execute-screen-options)
 
-  ;; Make A-- : [Alt] + [-] (in addition to / instead of  "_" = (on US-keyboard) [Shift]+ [-]
-  (define-key ess-mode-map          [?\A--] 'ess-insert-S-assign)
-  (define-key inferior-ess-mode-map [?\A--] 'ess-insert-S-assign)
+  ;; Make M-- : [Alt] + [-] (in addition to / instead of  "_" = (on US-keyboard) [Shift]+ [-]
+  ;; Note this overwrites 'M--' as "negative argument" (still on 'C--'):
+  (define-key ess-mode-map          [?\M--] 'ess-insert-S-assign)
+  (define-key inferior-ess-mode-map [?\M--] 'ess-insert-S-assign)
   )
 
 
@@ -769,6 +793,68 @@ and I need to relearn emacs lisp (but I had to, anyway."
               (local-set-key "\M-\r" 'ess-use-this-dir))
             ))
 
+
+
+;;; S imenu support
+
+;; don't use syntax classes, bad for etags
+(defvar ess-imenu-S-generic-expression
+  '(("Functions" "^\\(.+\\)[ \t\n]*<-[ \t\n]*function[ ]*(" 1)
+    ("Classes" "^.*setClass(\\(.*\\)," 1)
+    ("Coercions" "^.*setAs(\\([^,]+,[^,]*\\)," 1) ; show from and to
+    ("Generics" "^.*setGeneric(\\([^,]*\\)," 1)
+    ("Methods" "^.*set\\(Group\\|Replace\\)?Method(\"\\(.+\\)\"," 2)
+    ;;[ ]*\\(signature=\\)?(\\(.*,?\\)*\\)," 1)
+    ;;
+    ;;("Other" "^\\(.+\\)\\s-*<-[ \t\n]*[^\\(function\\|read\\|.*data\.frame\\)]" 1)
+    ("Package" "^.*\\(library\\|require\\)(\\(.*\\)" 2)
+    ("Data" "^\\(.+\\)[ \t\n]-*<-[ \t\n]*\\(read\\|.*data\.frame\\).*(" 1)))
+
+(defun ess-imenu-S (&optional arg)
+  "S Language Imenu support for ESS."
+  (interactive)
+  (setq imenu-generic-expression ess-imenu-generic-expression)
+  (imenu-add-to-menubar "Imenu-S"))
+
+(defalias 'ess-imenu-R 'ess-imenu-S)
+
+
+ ;;; Speedbar stuff.
+(defun ess-S-initialize-speedbar ()
+  "Extend to all extensions; see initialization, and edit."
+  (speedbar-add-supported-extension ".R")
+  (speedbar-add-supported-extension ".S")
+  (speedbar-add-supported-extension ".s")
+  (speedbar-add-supported-extension ".q"))
+
+                                        ;(if (featurep 'speedbar)
+                                        ;    (progn
+                                        ;      (message "enabling speedbar support")
+                                        ;      (require 'speedbar)
+                                        ;      (ess-S-initialize-speedbar)))
+
+(eval-when-compile
+  (condition-case nil
+      (progn
+        (require 'speedbar)
+        (when (featurep 'speedbar)
+          (message "enabling speedbar support")
+
+          (defun S-speedbar-buttons (buffer)
+            "attempted hack."
+
+            ;;(speedbar-make-tag-line)
+            ;;(speedbar-insert-button)
+            (speedbar-with-writable))
+
+          (fset 'R-speedbar-buttons 'S-speedbar-buttons)
+
+          (defun S-speedbar-menu-items  ( )
+            "Need to write.")
+
+          (ess-S-initialize-speedbar)))
+    (error nil)))
+
 (provide 'ess-s-l)
 
  ; Local variables section
