@@ -1,6 +1,6 @@
 ;;; ess-sas-l.el --- SAS customization
 
-;; Copyright (C) 1997--2009 A.J. Rossini, Rich M. Heiberger, Martin
+;; Copyright (C) 1997--2009 A.J. Rossini, Richard M. Heiberger, Martin
 ;;      Maechler, Kurt Hornik, Rodney Sparapani, and Stephen Eglen.
 
 ;; Authors: Richard M. Heiberger
@@ -103,7 +103,7 @@ the mode line."
   (SAS-mode)
   (setq mode-name "ESS[LOG]")
   (ess-transcript-minor-mode 1)
-  (toggle-read-only t)) ;; to protect the buffer.
+  (setq buffer-read-only t)) ;; to protect the buffer.
 
 (defun SAS-listing-mode()
   "Fundamental mode with `ess-listing-minor-mode' and read-only."
@@ -112,7 +112,7 @@ the mode line."
   (setq mode-name "ESS[LST]")
   (ess-listing-minor-mode 1)
   (use-local-map sas-mode-local-map)
-  (toggle-read-only t)) ;; to protect the buffer.
+  (setq buffer-read-only t)) ;; to protect the buffer.
 
 (fset 'sas-log-mode        'SAS-log-mode)
 (fset 'SAS-transcript-mode 'SAS-log-mode)
@@ -250,7 +250,7 @@ number."
 ;;; ...
 
 
-(defvar SAS-mode-font-lock-keywords
+(defvar SAS-mode-font-lock-defaults
   (if ess-sas-run-regexp-opt
       (list
        ;; .log NOTE: messages
@@ -415,7 +415,7 @@ number."
                                    "options"
                                    "plot" "pmenu" "print" "printto"
                                    "rank" "registry" "report"
-                                   "sgpanel" "sgplot" "sgscatter" "sort" "sql" "standard" "summary"
+                                   "setinit" "sgpanel" "sgplot" "sgscatter" "sort" "sql" "standard" "summary"
                                    "tabulate" "template" "timeplot" "transpose" "trantab"
                                    "univariate"
 
@@ -777,9 +777,10 @@ number."
     (tab-stop-list                . ess-sas-tab-stop-list)
     (ess-mode-syntax-table        . SAS-syntax-table)
     (font-lock-keywords-case-fold-search . t)
-    (font-lock-defaults           . '(SAS-mode-font-lock-keywords)))
+    (font-lock-defaults           . '(SAS-mode-font-lock-defaults)))
   "General options for editing SAS source files.")
 
+
 (defun beginning-of-sas-statement (arg &optional comment-start)
   "Move point to beginning of current sas statement."
   (interactive "P")
@@ -991,7 +992,7 @@ opening /* appears.  returns 0 otherwise."
 Proc Lifetest.  Operates on current region.  A major space saver if there is
 heavy censoring."
   (interactive)
-  (if buffer-read-only (toggle-read-only))
+  (if buffer-read-only (setq buffer-read-only nil))
   (goto-char (point-min))
   (while (re-search-forward "^.*[ ]+[.][ ]+[.][ ]+[.][ ]+.*$" nil t)
     (replace-match "" nil nil)))
@@ -1555,7 +1556,7 @@ page ;
   (setq major-mode 'sas-dir-mode)
   (setq mode-name "SAS")
   (setq sas-directory-name (expand-file-name default-directory))
-  (toggle-read-only 1))
+  (setq buffer-read-only 1))
 
 
 ;;(defun sas-make-library (directory &optional update)
@@ -1828,6 +1829,14 @@ whose beginning matches the regexp `page-delimiter'."
                                                        (match-end 2)))))
          str)))
 
+
+(defun ess-imenu-SAS (&optional arg)
+  "SAS language Imenu support for ESS."
+  (interactive)
+  (setq imenu-generic-expression
+        '( (nil "[ \t\n=]\\([a-zA-Z_][a-zA-Z_0-9]*[.][a-zA-Z_][a-zA-Z_0-9]*\\)[ ,()\t\n;]" 1)))
+  (imenu-add-to-menubar "SAS Datasets"))
+
 ;;(defun sas-revert-library ()
 ;;  "Update current library."
 ;;  (interactive)
