@@ -1,7 +1,7 @@
 ;;; ess-site.el --- user customization of ESS
 
 ;; Copyright (C) 1993 David M. Smith
-;; Copyright (C) 1997--2011 A.J. Rossini, Richard M. Heiberger, Martin
+;; Copyright (C) 1997--2012 A.J. Rossini, Richard M. Heiberger, Martin
 ;;      Maechler, Kurt Hornik, Rodney Sparapani, and Stephen Eglen.
 
 ;; Author: David Smith <D.M.Smith@lancaster.ac.uk>
@@ -58,6 +58,7 @@
 
 ;; provide here; otherwise we'll get infinite loops of (require ..):
 (provide 'ess-site)
+;;(require 'ess-sp6-d)
 
 ;;;; 1. Load path, autoloads, and major modes
 ;;;; ========================================
@@ -183,6 +184,7 @@ for ESS, such as icons.")
 ;;   "*Location of the ESS info/ directory.
 ;; The ESS info directory stores the ESS info files.")
 
+
 ;;(1.2) If ess.info is not found, then ess-lisp-directory/../doc/info is added
 ;; resurrecting Stephen's version with a bug-fix & xemacs compatibility
 (unless (locate-file "ess.info"
@@ -193,6 +195,13 @@ for ESS, such as icons.")
                    'Info-directory-list 'Info-default-directory-list)
                (expand-file-name "../doc/info/" ess-lisp-directory)))
 
+
+;; ALWAYS:
+(ess-message "[ess-site:] require 'ess   *ITSELF* ...")
+(require 'ess); -> loads ess-custom.el and more
+(ess-message "[ess-site:] .. after requiring 'ess ...")
+
+
 ;;; (1.3) Files ending in .q and .S are considered to be S source files
 ;;; Files ending in .St are considered to be S transcript files
 ;;;
@@ -238,6 +247,7 @@ between .s or .S files and assembly mode.
            ("\\.[sS]nw\\'"      . Snw-mode); currently identical to Rnw-mode
            ("\\.[rR]profile\\'" . R-mode)
            ("NAMESPACE\\'"      . R-mode)
+           ("CITATION\\'"       . R-mode)
            ("\\.omg\\'"         . omegahat-mode)
            ("\\.hat\\'"         . omegahat-mode) ;; Duncan's pref'd...
            ("\\.lsp\\'"         . XLS-mode)
@@ -289,8 +299,8 @@ between .s or .S files and assembly mode.
 ;;(setq-default inferior-S4-program-name "/disk05/s4/S")
 ;;(setq-default inferior-S+4-program-name "Splus")
 ;;(setq-default inferior-S+5-program-name "Splus5")
-;;(setq-default inferior-S+6-program-name "Splus7") ; unix systems
-;;(setq-default inferior-S+6-program-name "Splus8") ; unix systems
+;;(setq-default inferior-S+-program-name "Splus7") ; unix systems ; or
+;;(setq-default inferior-S+-program-name "Splus") ; unix systems
 ;;
 ;; If you wish to call other versions of R on a Unix system, ESS
 ;; should auto-detect other versions of R, according to matches to the
@@ -311,7 +321,7 @@ between .s or .S files and assembly mode.
 ;;; to emacs, giving the user the opportunity to
 ;;; (1) edit the output into a clean ess-transcript file before printing, or
 ;;; (2) print a region of the file.
-;;(setq-default inferior-S+4-print-command "S_PRINT_COMMAND=gnuclientw.exe")
+;;(setq-default inferior-S+4-print-command "S_PRINT_COMMAND=emacsclientw.exe")
 
 ;;; The editor and pager output from S+4 and Sqpe+4 are sent by
 ;;; StatSci default to notepad, effectively using the definition:
@@ -321,7 +331,7 @@ between .s or .S files and assembly mode.
 ;;; ESS sends the output from both commands to an emacs buffer using
 ;;; the definition:
 ;;(setq-default  inferior-S+4-editor-pager-command
-;;   "options(editor='gnuclient.exe', pager='gnuclientw.exe')")
+;;   "options(editor='emacsclient.exe', pager='emacsclientw.exe')")
 
 ;;; These commands are for running the PC version of Sqpe of S+4 and
 ;;; S+6 in an emacs buffer, using the same technology as ESS uses for
@@ -365,12 +375,11 @@ sending `inferior-ess-language-start' to S-Plus.")
 
 
 ;; (1.5) Require the needed dialects for your setup.
-(if (< max-specpdl-size 700)     ;;; ESS won't load at the default of 600
-    (setq max-specpdl-size 700))
 
 (ess-message "[ess-site:] Before requiring dialect 'ess-*-d ....")
 (ess-message "[ess-site:] require 'ess-r-d ...")
 (require 'ess-r-d)    ;; R
+(require 'ess-julia)
 ;; (ess-message "[ess-site:] require 'ess-s4-d ...")
 ;; (require 'ess-s4-d) ; has become VERY RARE ..
 
@@ -418,11 +427,6 @@ sending `inferior-ess-language-start' to S-Plus.")
 (require 'ess-noweb)
 (require 'ess-swv); for Sweave
 
-;; ALWAYS:
-(ess-message "[ess-site:] require 'ess   *ITSELF* ...")
-(require 'ess); -> loads ess-custom.el and more
-(ess-message "[ess-site:] .. after requiring 'ess ...")
-
 (ess-write-to-dribble-buffer
  (format "[ess-site.el _2_]: ess-customize-alist=%s \n"
          ess-customize-alist))
@@ -459,22 +463,16 @@ sending `inferior-ess-language-start' to S-Plus.")
 (autoload 'ess-rdired "ess-rdired"
   "View *R* objects in a dired-like buffer." t)
 
-(autoload 'ess-roxy-mode "ess-roxy"
-  "Insert and edit Roxygen tags for function definitions." t)
-;; if ever ess-roxy works for non- R ess modes, we will have
-;; (add-hook 'ess-mode-hook 'ess-roxy-mode)
-(add-hook 'R-mode-hook 'ess-roxy-mode)
 
-
-;;; On a PC, the default is S+6.
-;; Elsewhere (unix and linux) the default is S+6
+;;; On a PC, the default is S+.
+;; Elsewhere (unix and linux) the default is S+
 (cond  (ess-microsoft-p
         ;; MS-Windows-------------------------------------------------
 
         ;;        (fset 'S
         ;;           (if (equal (file-name-nondirectory shell-file-name) "cmdproxy.exe")
-        ;;               'S+6-msdos
-        ;;             'S+6))
+        ;;               'S+-msdos
+        ;;             'S+))
         (defun S-by-icon (&rest x)
           (interactive)
           (message "Please start S+ from the icon.
@@ -483,17 +481,17 @@ sending `inferior-ess-language-start' to S-Plus.")
         (fset 'S 'S-by-icon)
         (fset 'S-existing
               (if (equal (file-name-nondirectory shell-file-name) "cmdproxy.exe")
-                  'S+6-msdos-existing
-                'S+6-existing))
-        (fset 'Sqpe 'Sqpe+6)
-        (fset 's-mode 'S+6-mode)
-        (fset 's-transcript-mode 'S+6-transcript-mode))
+                  'S+-msdos-existing
+                'S+-existing))
+        (fset 'Sqpe 'Sqpe+)
+        (fset 's-mode 'S+-mode)
+        (fset 's-transcript-mode 'S+-transcript-mode))
 
        (t ;;((eq system-type 'gnu/linux)
         ;; Linux etc (including Mac OSX !?) --------------------------
-        (fset 'S 'S+6)
-        (fset 's-mode 'S+6-mode)
-        (fset 's-transcript-mode 'S+6-transcript-mode)))
+        (fset 'S 'S+)
+        (fset 's-mode 'S+-mode)
+        (fset 's-transcript-mode 'S+-transcript-mode)))
 
 
 ;;;;* Alias S-mode to s-mode
@@ -530,28 +528,37 @@ sending `inferior-ess-language-start' to S-Plus.")
                                    (ess-sqpe-versions-create ess-SHOME-versions-64 "-64-bit")) ;; 64-bit
                                 (ess-s-versions-create)))) ;; use ess-s-versions
   (ess-message "[ess-site:] (let ... after (ess-s-versions-create) ...")
+
   (if ess-microsoft-p
       (setq ess-rterm-version-paths ;; (ess-find-rterm))
             (ess-flatten-list
              (ess-uniq-list
-              (if (getenv "ProgramW6432")
-                  (let ((P-1 (getenv "ProgramFiles(x86)"))
-                        (P-2 (getenv "ProgramW6432")))
-                    (nconc
-                     ;; always 32 on 64 bit OS, nil on 32 bit OS
-                     (ess-find-rterm (concat P-1 "/R/") "bin/Rterm.exe")
-                     (ess-find-rterm (concat P-1 "/R/") "bin/i386/Rterm.exe")
-                     ;; keep this both for symmetry and because it can happen:
-                     (ess-find-rterm (concat P-1 "/R/") "bin/x64/Rterm.exe")
+              (if (not ess-directory-containing-R)
+                  (if (getenv "ProgramW6432")
+                      (let ((P-1 (getenv "ProgramFiles(x86)"))
+                            (P-2 (getenv "ProgramW6432")))
+                        (nconc
+                         ;; always 32 on 64 bit OS, nil on 32 bit OS
+                         (ess-find-rterm (concat P-1 "/R/") "bin/Rterm.exe")
+                         (ess-find-rterm (concat P-1 "/R/") "bin/i386/Rterm.exe")
+                         ;; keep this both for symmetry and because it can happen:
+                         (ess-find-rterm (concat P-1 "/R/") "bin/x64/Rterm.exe")
 
-                     ;; always 64 on 64 bit OS, nil on 32 bit OS
-                     (ess-find-rterm (concat P-2 "/R/") "bin/Rterm.exe")
-                     (ess-find-rterm (concat P-2 "/R/") "bin/i386/Rterm.exe")
-                     (ess-find-rterm (concat P-2 "/R/") "bin/x64/Rterm.exe")
-                     ))
-                (let ((PF (getenv "ProgramFiles")))
+                         ;; always 64 on 64 bit OS, nil on 32 bit OS
+                         (ess-find-rterm (concat P-2 "/R/") "bin/Rterm.exe")
+                         (ess-find-rterm (concat P-2 "/R/") "bin/i386/Rterm.exe")
+                         (ess-find-rterm (concat P-2 "/R/") "bin/x64/Rterm.exe")
+                         ))
+                    (let ((PF (getenv "ProgramFiles")))
+                      (nconc
+                       ;; always 32 on 32 bit OS, depends on 32 or 64 process on 64 bit OS
+                       (ess-find-rterm (concat PF "/R/") "bin/Rterm.exe")
+                       (ess-find-rterm (concat PF "/R/") "bin/i386/Rterm.exe")
+                       (ess-find-rterm (concat PF "/R/") "bin/x64/Rterm.exe")
+                       ))
+                    )
+                (let ((PF ess-directory-containing-R))
                   (nconc
-                   ;; always 32 on 32 bit OS, depends on 32 or 64 process on 64 bit OS
                    (ess-find-rterm (concat PF "/R/") "bin/Rterm.exe")
                    (ess-find-rterm (concat PF "/R/") "bin/i386/Rterm.exe")
                    (ess-find-rterm (concat PF "/R/") "bin/x64/Rterm.exe")
@@ -613,9 +620,6 @@ sending `inferior-ess-language-start' to S-Plus.")
   (add-hook 'Rd-mode-hook 'turn-on-font-lock t)
   (add-hook 'inferior-ess-mode-hook 'turn-on-font-lock t))
 
-;; If nil, then don't font-lock the input
-;; if t, font-lock (default).
-(setq inferior-ess-font-lock-input t) ; from RMH
 
 ;;; (3.2) Framepop.  Windows produced by ess-execute-objects etc. are
 ;;; often unnecessarily large. The framepop package makes such
@@ -684,7 +688,7 @@ sending `inferior-ess-language-start' to S-Plus.")
   (add-hook 'ess-post-run-hook
     (lambda()
       (when (string= ess-dialect "R")
-        (ess-eval-linewise "options(chmhelp = FALSE, help_type = \"text\")"
+        (ess-eval-linewise "options(chmhelp=FALSE, help_type=\"text\")"
                            nil nil nil 'wait)))))
 
 
