@@ -1,7 +1,7 @@
 ;;; ess-sp5-d.el --- S-plus 5  customization
 
 ;; Copyright (C) 1998 A.J. Rossini
-;; Copyright (C) 1999--2004 A.J. Rossini, Rich M. Heiberger, Martin
+;; Copyright (C) 1999--2004 A.J. Rossini, Richard M. Heiberger, Martin
 ;;      Maechler, Kurt Hornik, Rodney Sparapani, and Stephen Eglen.
 
 ;; Author: A.J. Rossini <rossini@biostat.washington.edu>
@@ -56,10 +56,10 @@
      (ess-object-name-db-file           . "ess-sp5-namedb.el")
      (inferior-ess-program              . inferior-S+5-program-name)
      ;;(inferior-ess-objects-pattern    . ".*") ; for new s4 stuff
-     (inferior-ess-help-command   . "help(\"%s\",pager=\"slynx -dump\",window=F)\n")
+     (inferior-ess-help-command   . "help(\"%s\", pager=\"slynx -dump\", window=FALSE)\n")
      (inferior-ess-help-filetype . nil)
      (inferior-ess-search-list-command  . "searchPaths()\n")
-     (inferior-ess-start-args      . inferior-Splus-args)
+     (inferior-ess-start-args      . inferior-S+-start-args)
      (ess-STERM  . "iESS")
      )
    S+common-cust-alist)
