@@ -1,7 +1,7 @@
 ;;; ess-trns.el --- Support for manipulating S transcript files
 
 ;; Copyright (C) 1989--1994 Bates, Kademan, Ritter and Smith
-;; Copyright (C) 1997--2010 A.J. Rossini, Rich M. Heiberger, Martin
+;; Copyright (C) 1997--2010 A.J. Rossini, Richard M. Heiberger, Martin
 ;;      Maechler, Kurt Hornik, Rodney Sparapani, and Stephen Eglen.
 ;; Copyright (C) 2011--2012 A.J. Rossini, Richard M. Heiberger, Martin Maechler,
 ;;      Kurt Hornik, Rodney Sparapani, Stephen Eglen and Vitalie Spinu.
@@ -84,8 +84,8 @@
 
   (define-key ess-transcript-mode-map "\C-c\C-j" 'ess-transcript-send-command)
   (define-key ess-transcript-mode-map "\C-c\M-j" 'ess-transcript-send-command-and-move)
-  (define-key ess-transcript-mode-map "\M-\C-a"  'ess-beginning-of-function)
-  (define-key ess-transcript-mode-map "\M-\C-e"  'ess-end-of-function)
+  (define-key ess-transcript-mode-map "\M-\C-a"  'ess-goto-end-of-function-or-para)
+  (define-key ess-transcript-mode-map "\M-\C-e"  'ess-goto-end-of-function-or-para)
   (define-key ess-transcript-mode-map "\C-c\C-y" 'ess-switch-to-ESS)
   (define-key ess-transcript-mode-map "\C-c\C-z" 'ess-switch-to-end-of-ESS)
   (define-key ess-transcript-mode-map "\C-c\C-v" 'ess-display-help-on-object)
@@ -167,7 +167,7 @@ in the region, leaving only the S commands.  Other keybindings are:
   (interactive)
   (require 'ess-inf)
   (kill-all-local-variables)
-  (toggle-read-only t) ;; to protect the buffer.
+  (setq buffer-read-only t) ;; to protect the buffer.
   (ess-setq-vars-local alist); (current-buffer))
   (setq major-mode 'ess-transcript-mode)
   (setq mode-name "ESS Transcript")
@@ -195,9 +195,13 @@ in the region, leaving only the S commands.  Other keybindings are:
   (setq comint-prompt-regexp (concat "^" inferior-ess-prompt))
 
   ;; font-lock support
-  (make-local-variable 'font-lock-defaults)
-  (setq font-lock-defaults
-        '(inferior-ess-font-lock-keywords nil nil ((?' . "."))))
+
+  (when inferior-ess-font-lock-keywords ;; new system
+    (setq inferior-ess-font-lock-defaults
+          (ess--extract-default-fl-keywords inferior-ess-font-lock-keywords)))
+  
+  (set (make-local-variable 'font-lock-defaults)
+       '(inferior-ess-font-lock-defaults nil nil ((?\. . "w") (?\_ . "w") (?' . "."))))
 
   ;;; Keep <tabs> out of the code.
   (make-local-variable 'indent-tabs-mode)
@@ -219,8 +223,7 @@ is not already."
     (if (get-buffer-window ess-buf) nil
       (display-buffer ess-buf t))
     (let ((input (inferior-ess-get-old-input)))
-      (save-excursion
-        (set-buffer ess-buf)
+      (with-current-buffer ess-buf
         (goto-char (point-max))
         (ess-eval-linewise input)))))
 
@@ -236,8 +239,7 @@ is not already."
     (if (get-buffer-window ess-buf) nil
       (display-buffer ess-buf t))
     (let ((input (inferior-ess-get-old-input)))
-      (save-excursion
-        (set-buffer ess-buf)
+      (with-current-buffer ess-buf
         (goto-char (point-max))
         (ess-eval-linewise input nil nil nil 1))))
   ;; replacement code ends
@@ -254,8 +256,7 @@ is not already."
     (setq ess-local-process-name proc)
     (if (get-buffer-window ess-buf) nil
       (display-buffer ess-buf t))
-    (save-excursion
-      (set-buffer ess-buf)
+    (with-current-buffer ess-buf
       (goto-char (point-max))
       (insert input)))
   (ess-switch-to-end-of-ESS))
@@ -263,8 +264,8 @@ is not already."
 (defun ess-transcript-clean-region (beg end even-if-read-only)
   "Strip the transcript in the region, leaving only (R/S/Lsp/..) commands.
 Deletes any lines not beginning with a prompt, and then removes the
-prompt from those lines that remain.  Prefix argument means to use
-\\[toggle-read-only] to clean even if the buffer is \\[read-only]."
+prompt from those lines that remain.  Prefix argument means to 
+clean even if the buffer is \\[read-only]."
   (interactive "r\nP")
   (unless inferior-ess-prompt
     (error "Cannot clean ESS transcript region in this mode!
@@ -274,7 +275,7 @@ prompt from those lines that remain.  Prefix argument means to use
   (let ((do-toggle (and buffer-read-only even-if-read-only))
         (ess-prompt-rx (concat "^" inferior-ess-prompt)))
     (save-excursion
-      (if do-toggle (toggle-read-only 0))
+      (if do-toggle (setq buffer-read-only 0))
       (save-restriction
         (unless (featurep 'xemacs) ;; does not exist in xemacs:
           (deactivate-mark))
@@ -286,7 +287,7 @@ prompt from those lines that remain.  Prefix argument means to use
         (while (re-search-forward ess-prompt-rx nil t)
           (replace-match "" nil nil)))
 
-      (if do-toggle (toggle-read-only 1)))))
+      (if do-toggle (setq buffer-read-only 1)))))
 
 
 ;; unfinished idea :-----------------------
