@@ -1,9 +1,9 @@
 ;;; ess.el --- Emacs Speaks Statistics: statistical programming within Emacs
 
 ;; Copyright (C) 1989--1996 Bates, Kademan, Ritter and Smith
-;; Copyright (C) 1997--2010 A.J. Rossini, Rich M. Heiberger, Martin
+;; Copyright (C) 1997--2010 A.J. Rossini, Richard M. Heiberger, Martin
 ;;      Maechler, Kurt Hornik, Rodney Sparapani, and Stephen Eglen.
-;; Copyright (C) 2011--2012 A.J. Rossini, Rich M. Heiberger, Martin Maechler,
+;; Copyright (C) 2011--2012 A.J. Rossini, Richard M. Heiberger, Martin Maechler,
 ;;      Kurt Hornik, Rodney Sparapani, Stephen Eglen and Vitalie Spinu.
 
 ;; Author: Doug Bates
@@ -108,9 +108,10 @@
     (require 'font-lock))
 
 (require 'ess-compat)
+(require 'ess-custom)
+(require 'ess-mode)
+(require 'ess-inf)
 
-(eval-and-compile
-  (require 'ess-custom))
 
  ; ess-mode: editing S/R/XLS/SAS source
 
@@ -169,8 +170,7 @@
               ;; then it has two lines that look like
               ;; |Revision: 4803
               ;; |Last Changed Date: 2012-04-16
-              (save-excursion
-                (set-buffer buffer)
+              (with-current-buffer buffer
                 (ess-write-to-dribble-buffer
                  (format "(ess-version-string): buffer=%s\n" (buffer-name (current-buffer))))
                 (goto-char (point-min))
@@ -274,23 +274,25 @@ See also `ess-use-ido'.
   "Load all the extra features depending on custom settings."
 
   (let ((mode (if inferior 'inferior-ess-mode 'ess-mode))
-        (Rp (string-match "^R" ess-dialect))
+        (isR (string-match "^R" ess-dialect))
         (emacsp (featurep 'emacs)))
 
     ;; auto-complete
-    (when (and emacsp Rp
-               (require 'auto-complete nil t)
+    (when (and emacsp 
+               (require 'auto-complete nil 'no-error)
                (if inferior
                    (eq ess-use-auto-complete t)
                  ess-use-auto-complete))
       (add-to-list 'ac-modes mode)
-      (mapcar (lambda (el) (add-to-list 'ac-trigger-commands el))
-              '(ess-smart-comma smart-operator-comma skeleton-pair-insert-maybe))
-      (setq ac-sources '(ac-source-R ac-source-filename)))
+      ;; (mapcar (lambda (el) (add-to-list 'ac-trigger-commands el))
+      ;;         '(ess-smart-comma smart-operator-comma skeleton-pair-insert-maybe))
+      (add-to-list 'ac-sources 'ac-source-filename)
+      (when isR 
+        (add-to-list 'ac-sources 'ac-source-R)))
 
     ;; eldoc)
     (require 'eldoc)
-    (when (and Rp
+    (when (and ess-funargs-command ;; if mode provide this, it suports eldoc
                (or (and (not inferior) ess-use-eldoc)
                    (and inferior (eq ess-use-eldoc t))))
       (when (> eldoc-idle-delay 0.4) ;; default is too slow for paren help
@@ -301,7 +303,7 @@ See also `ess-use-ido'.
         ))
 
     ;; tracebug
-    (when (and ess-use-tracebug emacsp inferior Rp)
+    (when (and ess-use-tracebug emacsp inferior isR)
       (ess-tracebug 1))
     ))
 
@@ -372,16 +374,26 @@ Otherwise try a list of fixed known viewers."
 (defun ess-get-pdf-viewer ()
   "Get external PDF viewer to be used from ESS.
 Use `ess-pdf-viewer-pref' when that is executably found by \\[executable-find].
-Otherwise try a list of fixed known viewers."
-  (file-name-nondirectory
-   (or (and ess-pdf-viewer-pref         ; -> ./ess-custom.el
-            (executable-find ess-pdf-viewer-pref))
-       (car (ess-get-words-from-vector
-             "getOption(\"pdfviewer\")\n"))
-       (executable-find "evince")
-       (executable-find "kpdf")
-       (executable-find "xpdf")
-       (executable-find "acroread"))))
+Otherwise try a list of fixed known viewers.
+"
+  (let ((viewer (or ess-pdf-viewer-pref
+                    ;; (and (stringp ess-pdf-viewer-pref)         ; -> ./ess-custom.el
+                    ;;      (executable-find ess-pdf-viewer-pref))
+                    (executable-find "evince")
+                    (executable-find "kpdf")
+                    (executable-find "okular")
+                    (executable-find "xpdf")
+                    (executable-find "acroread")
+                    (executable-find "xdg-open")
+                    ;; this one is wrongly wrong, (ok for time being as it is use donly in swv)
+                    (car (ess-get-words-from-vector
+                          "getOption(\"pdfviewer\")\n"))
+                    )))
+    (when (stringp viewer)
+      (setq viewer (file-name-nondirectory viewer)))
+    viewer))
+
+        
 
 
 
@@ -411,20 +423,30 @@ Otherwise try a list of fixed known viewers."
   (let (deactivate-mark)
     (with-current-buffer ess-dribble-buffer
       (goto-char (point-max))
-      (insert text))))
+      (insert-before-markers text))))
 
 ;; Shortcut to render "dribbling" statements less cluttering:
 (defun ess-if-verbose-write (text)
   "Write TEXT to dribble buffer ('*ESS*') only *if* `ess-verbose'."
   (if ess-verbose (ess-write-to-dribble-buffer text)))
 
+
+(defvar ess--make-local-vars-permenent nil
+  "If this varialbe is non-nil in a buffer make all variable permannet.
+Used in noweb modes.")
+(make-variable-buffer-local 'ess--make-local-vars-permenent)
+(put 'ess--make-local-vars-permenent 'permanent-local t)
+
 (defun ess-setq-vars-local (alist &optional buf)
   "Set language variables from ALIST, in buffer BUF, if desired."
   (if buf (set-buffer buf))
+  ;; (setq alist (reverse alist)) ;; It should really be in reverse order; 
   (mapc (lambda (pair)
           (make-local-variable (car pair))
-          (if (cdr pair)
-              (set (car pair) (eval (cdr pair)))))
+          (set (car pair) (eval (cdr pair)))
+          (when ess--make-local-vars-permenent
+            (put (car pair) 'permanent-local t)) ;; hack for Rnw
+          )
         alist)
   (ess-write-to-dribble-buffer
    (format "(ess-setq-vars-LOCAL): language=%s, dialect=%s, buf=%s, comint..echoes=%s, comint..sender=%s\n"
