@@ -1,10 +1,10 @@
 ;;; essd-els.el --- S-PLUS 3.x at another location customization
 
 ;; Copyright (C) 1998 Richard M. Heiberger
-;; Copyright (C) 1999--2005 A.J. Rossini, Rich M. Heiberger, Martin
+;; Copyright (C) 1999--2005 A.J. Rossini, Richard M. Heiberger, Martin
 ;;      Maechler, Kurt Hornik, Rodney Sparapani, and Stephen Eglen.
 
-;; Author: Richard M. Heiberger <rmh@fisher.stat.temple.edu>
+;; Author: Richard M. Heiberger <rmh@temple.edu>
 ;; Created: December 1998
 
 ;; Maintainer: ESS-core <ESS-core@r-project.org>
@@ -48,7 +48,7 @@
      (ess-loop-timeout                  . ess-S-loop-timeout);fixme: dialect spec.
      (ess-object-name-db-file           . "ess-spelsewhere-namedb.el" )
      (inferior-ess-program              . inferior-S-elsewhere-program-name)
-     (inferior-ess-help-command         . "help(\"%s\",pager=\"cat\",window=F)\n")
+     (inferior-ess-help-command         . "help(\"%s\", pager=\"cat\", window=F)\n")
 
      (inferior-ess-start-file           . nil) ;"~/.ess-S+3")
      (inferior-ess-start-args           . "-i")
@@ -102,8 +102,8 @@ return new alist whose car is the new pair and cdr is ALIST.
 (defun ess-select-alist-dialect ()
   "Query user for an ESS dialect and return the matching customize-alist."
   (interactive)
-  (let* ((dialects '("arc" "vst" "omg" "s3"  "s4" "stata" "r" "sp3" "sp4"
-                     "sqpe4" "sp5" "sp6" "sqpe6" "xls" "sas"))
+  (let* ((dialects '("R" "S+" "julia" "arc" "vst" "omg" "s3" "s4" "stata" "sp3" "sp4"
+                     "sqpe4" "sp5" "sqpe" "XLS" "SAS"))
          (dialect (ess-completing-read "Dialect" dialects nil t)))
     (cond
      ((string= dialect "arc")   ARC-customize-alist)
@@ -112,36 +112,36 @@ return new alist whose car is the new pair and cdr is ALIST.
      ((string= dialect "s3")    S3-customize-alist)
      ((string= dialect "s4")    S4-customize-alist)
      ((string= dialect "stata") STA-customize-alist)
-     ((string= dialect "r")     R-customize-alist )
+     ((string= dialect "R")     R-customize-alist )
      ((string= dialect "sp3")   S+3-customize-alist)
      ((string= dialect "sp4")   S+4-customize-alist)
      ((string= dialect "sqpe4") Sqpe+4-customize-alist)
      ((string= dialect "sp5")   S+5-customize-alist)
-     ((string= dialect "sp6")   S+6-customize-alist)
-     ((string= dialect "sqpe6") Sqpe+6-customize-alist)
-     ((string= dialect "xls")   XLS-customize-alist)
-     ((string= dialect "sas")   SAS-customize-alist);was S+elsewhere-customize-alist?
+     ((string= dialect "S+")    S+-customize-alist)
+     ((string= dialect "sqpe") Sqpe+-customize-alist)
+     ((string= dialect "XLS")   XLS-customize-alist)
+     ((string= dialect "SAS")   SAS-customize-alist);was S+elsewhere-customize-alist?
      (t                         S+elsewhere-customize-alist)
      )))
 
 
-(defun ESS-elsewhere (&optional proc-name)
-  "Call an inferior process from ELSEWHERE.
-This command is obsolete; please use `ess-remote' instead."
-  (interactive)
-  ;; Need to select a elsewhere-customize-alist
-  (let ((elsewhere-customize-alist (ess-select-alist-dialect)))
-    (ess-change-alist 'inferior-ess-program
-                      inferior-ESS-elsewhere-program-name
-                      elsewhere-customize-alist)
-    (setq ess-customize-alist elsewhere-customize-alist)
-    (ess-write-to-dribble-buffer
-     (format "\n(ESS-elsewhere): ess-dialect=%s, buf=%s\n" ess-dialect
-             (current-buffer)))
-    (inferior-ess)
-    (if (equal ess-language "S")
-        (if inferior-ess-language-start
-            (ess-eval-linewise inferior-ess-language-start)))))
+;; (defun ESS-elsewhere (&optional proc-name)
+;;   "Call an inferior process from ELSEWHERE.
+;; This command is obsolete; please use `ess-remote' instead."
+;;   (interactive)
+;;   ;; Need to select a elsewhere-customize-alist
+;;   (let ((elsewhere-customize-alist (ess-select-alist-dialect)))
+;;     (ess-change-alist 'inferior-ess-program
+;;                       inferior-ESS-elsewhere-program-name
+;;                       elsewhere-customize-alist)
+;;     (setq ess-customize-alist elsewhere-customize-alist)
+;;     (ess-write-to-dribble-buffer
+;;      (format "\n(ESS-elsewhere): ess-dialect=%s, buf=%s\n" ess-dialect
+;;              (current-buffer)))
+;;     (inferior-ess)
+;;     (if (equal ess-language "S")
+;;         (if inferior-ess-language-start
+;;             (ess-eval-linewise inferior-ess-language-start)))))
 
 
 (defun ess-add-ess-process ()
@@ -154,6 +154,7 @@ buffer on the local computer."
   (let ((proc (get-buffer-process (buffer-name))))
     (if (not proc)
         (error "No process is associated with this buffer.")
+      (set-process-filter proc 'inferior-ess-output-filter)
       (setq ess-current-process-name (process-name proc))
       (add-to-list 'ess-process-name-list
                    (list ess-current-process-name)))))
@@ -190,19 +191,24 @@ C-n to send lines over.  With SAS, use C-c i
     (inferior-ess-mode)
     (setq ess-local-process-name (or proc-name ess-current-process-name))
     (goto-char (point-max))
-    (if (equal ess-language "S")
-        (if inferior-ess-language-start
-            (progn
-              ;; FIXME hack (not in line with using ess-customize-alist)
-              (setq ess-editor nil)
-              (setq ess-pager nil)
-              (setq inferior-ess-language-start (eval inferior-S-language-start))
-              (ess-eval-linewise inferior-ess-language-start))))
+    (if inferior-ess-language-start
+        (ess-eval-linewise inferior-ess-language-start
+                           nil nil nil 'wait-prompt))
+
+    ;; todo: this is ugly, add to customise alist
+    (if (and (equal ess-dialect "R") (ess-current-R-at-least '2.7.0))
+        (ess-command ess--R-injected-code))
+
+    ;; (ess-load-extras t) ;; not working
+
+    (if (equal ess-dialect "S+")
+        (ess-command ess-S+--injected-code))
+
     (if (equal ess-language "SAS")
         (progn (font-lock-mode 0)
                (SAS-log-mode)
                (shell-mode)
-               (toggle-read-only nil)
+               (setq buffer-read-only nil)
                (font-lock-mode 1)))))
 
 
